{% extends "default/module.html.jinja2" %}

{% block nav %}
<style>
    .logo img {
        width: 100%;
        max-width: 100px;
        height: auto;
        display: block;
        margin: 0 auto 20px;
    }
</style>
<div class="logo">
    <a href="https://github.com/Jebel-Quant/rhiza" title="Home">
        <img src="https://raw.githubusercontent.com/Jebel-Quant/rhiza/main/assets/rhiza-logo.svg" alt="Rhiza Logo">
    </a>
</div>
{{ super() }}
{% endblock nav %}